function [SFdyn,SEdyn,Qdyn]=obrmap(PLANT,K,G,Q)
%
% OBRMAP(PLANT,K,G,Q)
% plots the closed-loop eigenvalues map of the  system PLANT in positive feedback
% with the observer-based controller defined by the Youla parameterization (K, G,
% Q(s)) associated with PLANT:
%     - K: state feedback gain, 
%     - G: state estimation gain, 
%     - Q(s): ss system of the dynamics Youla parameter.
%
% On the map, are marked:
%     - state feedback dynamics: spec(A-B*K) (blue o),
%     - state estimation dynamics: spec(A-G*C) (red o),
%     - Youla parameter dynamics:  spec(AQ) (green o).
% and also :
%     - open-loop plant dynamics: spec(A) (blue +),
%     - open-loop controller dynamics: (green +).
%
% Remark: * continuous-time formulation (or predictor form in
% discrete-time) for the state estimator
%
% [SFDYN,SEDYN,QDYN]=OBRMAP(PLANT,K,G,Q) does not open a graphics window
% but returns:
%   * SFDYN: state feedback dynamics: spec(A-B*K),
%   * SEDYN: state estimation dynamics: spec(A-G*C),
%   * QDYN: Youla parameter dynamics spec(Q.a);
%
% see also:  OBR2COR, COR2OBR, COR2OBRA

% D. Alazard /01/01/99
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.

[A,B,C,D]=ssdata(PLANT);
[p,m]=size(D);
SYS_K=ss([A-B*K-G*C+G*D*K],[G B-G*D],[-K;-C+D*K],...
         [0*ones(m,p) eye(m,m);eye(p,p) -D]);
SYS_K=lft(SYS_K,Q);

vbf=eig(feedback(PLANT,-SYS_K));
SFdyn=eig(A-B*K);
SEdyn=eig(A-G*C);
Qdyn=eig(Q.a);
if nargout==0,
    va=eig(A);
    vak=eig(SYS_K.a);
    figure
    plot(real(vbf),imag(vbf),'kx');
    hold on
    plot(real(SFdyn),imag(SFdyn),'bo','LineWidth',2);
    plot(real(SEdyn),imag(SEdyn),'ro','LineWidth',2);
    plot(real(Qdyn),imag(Qdyn),'go','LineWidth',2);
    plot(real(va),imag(va),'b+','LineWidth',2);
    plot(real(vak),imag(vak),'g+','LineWidth',2);
    if PLANT.Ts==0, sgrid; else zgrid;end;
    zoom on
end

